@@ -7,6 +7,7 @@ from mpl_toolkits.mplot3d.art3d import Line3DCollection
 import vtk
 from scipy import optimize
 from tqdm import tqdm
+import pyvista as pv
 
 from ..branch_addition.basis import tangent_basis
 
@@ -106,7 +107,7 @@ def get_longest_path(data,seed_edge):
     return paths[path_lengths.index(max(path_lengths))]
 
 def get_alternate_path(data,seed_edge,reference=None):
-    print('reference: {}'.format(reference))
+    #print('reference: {}'.format(reference))
     if reference is None:
         reference = get_longest_path(data,seed_edge)
     else:
@@ -135,7 +136,7 @@ def get_branches(data):
     counter = [len(path[:-1])]
     idx = 0
     while len(upcoming_evaluations) > 0:
-        print(upcoming_evaluations)
+        #print(upcoming_evaluations)
         if data[upcoming_evaluations[-1],15] > -1 and data[upcoming_evaluations[-1],16] > -1:
             pass
         else:
@@ -175,18 +176,47 @@ def get_points(data,branches):
                 branch_points.insert(0,data[edge,3:6].tolist())
             elif edge == path[0] and edge == 0 and not primed:
                 branch_points.insert(0,data[edge,3:6].tolist())
+                three_fourths = data[edge,0:3]*(1/4) + data[edge,3:6]*(3/4) #new
+                branch_points.insert(0,three_fourths.tolist()) #new
                 mid_point = (data[edge,0:3] + data[edge,3:6])/2
                 branch_points.insert(0,mid_point.tolist())
+                one_fourth = data[edge,0:3]*(3/4) + data[edge,3:6]*(1/4) #new
+                branch_points.insert(0,one_fourth.tolist()) #new
                 branch_points.insert(0,data[edge,0:3].tolist())
                 primed = True
             elif len(branch_points) == 0:
                 branch_points.insert(0,data[edge,3:6].tolist())
+                three_fourths = data[edge,0:3]*(1/4) + data[edge,3:6]*(3/4) #new
+                branch_points.insert(0,three_fourths.tolist()) #new
                 mid_point = (data[edge,0:3] + data[edge,3:6])/2
                 branch_points.insert(0,mid_point.tolist())
+                one_fourth = data[edge,0:3]*(3/4) + data[edge,3:6]*(1/4) #new
+                branch_points.insert(0,one_fourth.tolist()) #new
             else:
                 branch_points.insert(0,data[edge,3:6].tolist())
+                three_fourths = data[edge,0:3]*(1/4) + data[edge,3:6]*(3/4) #new
+                branch_points.insert(0,three_fourths.tolist()) #new
                 mid_point = (data[edge,0:3] + data[edge,3:6])/2
                 branch_points.insert(0,mid_point.tolist())
+                one_fourth = data[edge,0:3]*(3/4) + data[edge,3:6]*(1/4) #new
+                branch_points.insert(0,one_fourth.tolist()) #new
+        path_points.append(branch_points)
+    return path_points
+
+def get_points_v1(data,branches):
+    path_points = []
+    primed = False
+    for path in branches:
+        branch_points = []
+        for edge in reversed(path):
+            if edge == path[-1]:
+                branch_points.insert(0,data[edge,3:6].tolist())
+            elif edge == path[0] and edge == 0 and not primed:
+                branch_points.insert(0,data[edge,3:6].tolist())
+                branch_points.insert(0,data[edge,0:3].tolist())
+                primed = True
+            else:
+                branch_points.insert(0,data[edge,3:6].tolist())
         path_points.append(branch_points)
     return path_points
 
@@ -202,13 +232,37 @@ def get_radii(data,branches):
                 branch_radii.insert(0,data[edge,21])
                 branch_radii.insert(0,data[edge,21])
                 branch_radii.insert(0,data[edge,21])
+                branch_radii.insert(0,data[edge,21]) #new
+                branch_radii.insert(0,data[edge,21]) #new
                 primed = True
             elif len(branch_radii) == 0:
                 branch_radii.insert(0,data[edge,21])
                 branch_radii.insert(0,data[edge,21])
+                branch_radii.insert(0,data[edge,21]) #new
+                branch_radii.insert(0,data[edge,21]) #new
             else:
                 branch_radii.insert(0,data[edge,21])
                 branch_radii.insert(0,data[edge,21])
+                branch_radii.insert(0,data[edge,21]) #new
+                branch_radii.insert(0,data[edge,21]) #new
+        path_radii.append(branch_radii)
+    return path_radii
+
+def get_radii_v1(data,branches):
+    path_radii = []
+    primed = False
+    for path in branches:
+        branch_radii = []
+        for edge in reversed(path):
+            if edge == path[-1]:
+                branch_radii.insert(0,data[edge,21])
+            elif edge == path[0] and edge == 0 and not primed:
+                branch_radii.insert(0,(data[edge,21]+data[previous,21])/2)
+                branch_radii.insert(0,data[edge,21])
+                primed = True
+            else:
+                branch_radii.insert(0,(data[edge,21]+data[previous,21])/2)
+            previous = edge
         path_radii.append(branch_radii)
     return path_radii
 
@@ -226,20 +280,44 @@ def get_normals(data,branches):
                 branch_normals.insert(0,mid_vector.tolist())
                 branch_normals.insert(0,data[edge,12:15].tolist())
                 branch_normals.insert(0,data[edge,12:15].tolist())
+                branch_normals.insert(0,data[edge,12:15].tolist()) #new
+                branch_normals.insert(0,data[edge,12:15].tolist()) #new
                 primed = True
             elif len(branch_normals) == 0:
                 branch_normals.insert(0,data[edge,12:15].tolist())
                 branch_normals.insert(0,data[edge,12:15].tolist())
+                branch_normals.insert(0,data[edge,12:15].tolist()) #new
+                branch_normals.insert(0,data[edge,12:15].tolist()) #new
                 vector_2 = data[edge,12:15]
             else:
                 vector_1 = data[edge,12:15]
                 mid_vector = (vector_1+vector_2)/2
                 branch_normals.insert(0,mid_vector.tolist())
+                branch_normals.insert(0,data[edge,12:15].tolist()) #new
+                branch_normals.insert(0,data[edge,12:15].tolist()) #new
                 branch_normals.insert(0,data[edge,12:15].tolist())
                 vector_2 = data[edge,12:15]
         path_normals.append(branch_normals)
     return path_normals
 
+def get_normals_v1(data,branches):
+    path_normals = []
+    primed = False
+    for path in branches:
+        branch_normals = []
+        for edge in reversed(path):
+            if edge == path[-1]:
+                branch_normals.insert(0,data[edge,12:15].tolist())
+            elif edge == path[0] and edge == 0 and not primed:
+                branch_normals.insert(0,((data[edge,12:15]+data[previous,12:15])/2).tolist())
+                branch_normals.insert(0,data[edge,12:15].tolist())
+                primed = True
+            else:
+                branch_normals.insert(0,((data[edge,12:15]+data[previous,12:15])/2).tolist())
+            previous = edge
+        path_normals.append(branch_normals)
+    return path_normals
+
 def plot_sv_data(data):
     branches = get_branches(data)
     points   = get_points(data,branches)
@@ -256,7 +334,9 @@ def plot_sv_data(data):
 def get_interpolated_sv_data(data):
     branches = get_branches(data)
     points   = get_points(data,branches)
+    #print("Points: {}".format(points[0]))
     radii    = get_radii(data,branches)
+    #print("Radii:  {}".format(radii[0]))
     normals  = get_normals(data,branches)
     path_frames = []
     for idx in range(len(branches)):
@@ -271,27 +351,34 @@ def get_interpolated_sv_data(data):
     interp_xyz = []
     interp_r   = []
     interp_n   = []
+    interp_xyzr = []
     for idx in range(len(branches)):
         p = np.array(points[idx]).T
         r = np.array(radii[idx]).T
         n = np.array(normals[idx]).T
         if len(points[idx]) == 2:
             interp_xyz.append(splprep(p,k=1,s=0))
-            r = np.vstack((interp_xyz[-1][1],r))
-            interp_r.append(splprep(r,k=1,s=0))
+            rr = np.vstack((interp_xyz[-1][1],r))
+            interp_r.append(splprep(rr,k=1,s=0))
+            xyzr = np.vstack((p,r))
+            interp_xyzr.append(splprep(xyzr,k=1,s=0))
             #interp_n.append(splprep(n,k=1,s=0))
         elif len(points[idx]) == 3:
             interp_xyz.append(splprep(p,k=2,s=0))
-            r = np.vstack((interp_xyz[-1][1],r))
-            interp_r.append(splprep(r,k=2,s=0))
+            rr = np.vstack((interp_xyz[-1][1],r))
+            interp_r.append(splprep(rr,k=2,s=0))
+            xyzr = np.vstack((p,r))
+            interp_xyzr.append(splprep(xyzr,k=2,s=0))
             #interp_n.append(splprep(n,k=2,s=0))
         else:
             interp_xyz.append(splprep(p,s=0))
-            r = np.vstack((interp_xyz[-1][1],r))
-            interp_r.append(splprep(r,s=0))
+            rr = np.vstack((interp_xyz[-1][1],r))
+            interp_r.append(splprep(rr,s=0))
+            xyzr = np.vstack((p,r))
+            interp_xyzr.append(splprep(xyzr,s=0))
             #interp_n.append(splprep(n,s=0))
 
-    return interp_xyz,interp_r,interp_n,path_frames,branches
+    return interp_xyz,interp_r,interp_n,path_frames,branches,interp_xyzr
 
 def get_sv_data(network):
     if isinstance(network,tree):
@@ -422,6 +509,7 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     if radius_buffer is not None:
         r1 += r1*radius_buffer
         r2 += r2*radius_buffer
+    """
     contour_1 = vtk.vtkRegularPolygonSource()
     contour_2 = vtk.vtkRegularPolygonSource()
     contour_1.SetCenter([x1,y1,z1])
@@ -446,7 +534,12 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     collision.SetCollisionModeToAllContacts()
     collision.GenerateScalarsOn()
     collision.Update()
-    if collision.GetNumberOfContacts() > 0:
+    """
+    contour_1 = pv.Disc(center=[x1,y1,z1],inner=0,outer=r1,normal=[nx1,ny1,nz1],r_res=5,c_res=20).triangulate()
+    contour_2 = pv.Disc(center=[x2,y2,z2],inner=0,outer=r2,normal=[nx2,ny2,nz2],r_res=5,c_res=20).triangulate()
+    collision,ncol = contour_1.collision(contour_2,contact_mode=0)
+    #if collision.GetNumberOfContacts() > 0:
+    if ncol > 0:
         """
         actors = []
         colors = vtk.vtkNamedColors()
@@ -505,46 +598,165 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     """
 def contour_check(interp_xyz,interp_r,t0,t1,radius_buffer,n_sides=4):
     tl  = (t1 - t0)/10
-    x1,y1,z1      = splev(t0,interp_xyz[0])
-    _,r1            = splev(t0,interp_r[0])
-    #x11,y11,z11,_,_,_,_   = splev(t0+tl,contours[0])
-    #nx1 = x11-x1
-    #ny1 = y11-y1
-    #nz1 = z11-z1
-    nx1,ny1,nz1 = splev(t0,interp_xyz[0],der=1)
+    x1,y1,z1,r1      = splev(t0,interp_xyz[0])
+    #_,r1            = splev(t0,interp_r[0])
+    nx1,ny1,nz1,_ = splev(t0,interp_xyz[0],der=1)
     n1  = np.array([nx1,ny1,nz1])
     mag = np.linalg.norm(n1,axis=0)
     n1 = n1/mag
-    #nx1 = -mag/nx1
-    #ny1 = -mag/ny1
-    #nz1 = -mag/nz1
-    x2,y2,z2            = splev(t1,interp_xyz[0])
-    _,r2                  = splev(t1,interp_r[0])
-    #x22,y22,z22,_,_,_,_   = splev(t1+tl,contours[0])
-    #nx2 = x22-x2
-    #ny2 = y22-y2
-    #nz2 = z22-z2
-    nx2,ny2,nz2 = splev(t1,interp_xyz[0],der=1)
+    x2,y2,z2,r2            = splev(t1,interp_xyz[0])
+    #_,r2                  = splev(t1,interp_r[0])
+    nx2,ny2,nz2,_ = splev(t1,interp_xyz[0],der=1)
     n2  = np.array([nx2,ny2,nz2])
     mag = np.linalg.norm(n2,axis=0)
     n2 = n2/mag
-    #nx2 = -mag/nx2
-    #ny2 = -mag/ny2
-    #nz2 = -mag/nz2
-    #if np.sum(np.isclose(n1-n2,0)) == 3:
-    #    return False
-    #elif np.isclose(nx1,0) and np.isclose(ny1,0):
-    #print('{} {} {}'.format(x1,y1,z1))
-    #print('{} {} {}'.format(x2,y2,z2))
-    #print()
     collision = contour_collision(x2,y2,z2,nx2,ny2,nz2,r2,x1,y1,z1,nx1,ny1,nz1,r1,radius_buffer,interp_xyz,t0,t1,n_sides=n_sides)
-    #print(collision)
     return collision
-    #else:
-    #    collision = contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buffer,contours,t0,t1,n_sides=n_sides)
-    #    #print(collision)
-    #    return collision
 
+def contour_check_all(interp_xyz,interp_r,radius_buffer):
+    ANGLE_THRESHOLD = 20
+    sample_t = []
+    #n = np.linspace(0,1,1000)
+    desc1 = 'Extracting Loft Contours'
+    desc1 = desc1+' '*(40-len(desc1))
+    for kdx in tqdm(range(len(interp_xyz)),bar_format='{l_bar}{bar:10}{r_bar}{bar:-10b}',position=2,leave=False,desc=desc1):
+        tmp_t = []
+        t_list = []
+        t_list = np.linspace(0,1,10*len(interp_xyz[kdx][1]))
+        #plot_path_contours(interp_xyz[kdx],interp_r[kdx],t_list)
+        normal_list = []
+        x,y,z,r = splev(0,interp_xyz[kdx][0])
+        previous = [x,y,z,r]
+        nx,ny,nz,_ = splev(0,interp_xyz[kdx][0],der=1)
+        normal     = np.array([nx,ny,nz])
+        normal     = normal/np.linalg.norm(normal)
+        normal_list.append(normal)
+        # Clean out all contours that provide little information
+        tmp_t.append(t_list[0])
+        desc2 = 'Cleaning Out Redundant Contours'
+        desc2 = desc2+' '*(40-len(desc2))
+        for ct in tqdm(t_list[1:-1],bar_format='{l_bar}{bar:10}{r_bar}{bar:-10b}',position=3,leave=False,desc=desc2):
+            x,y,z,r = splev(ct,interp_xyz[kdx][0])
+            nx,ny,nz,_ = splev(ct,interp_xyz[kdx][0],der=1)
+            normal     = np.array([nx,ny,nz])
+            normal     = normal/np.linalg.norm(normal)
+            angle      = abs((np.arccos(np.dot(normal_list[-1],normal.T))/np.pi)*180)
+            dist       = ((previous[0]-x)**2+(previous[1]-y)**2+(previous[2]-z)**2)**(1/2)
+            if angle > ANGLE_THRESHOLD or (dist>(r+previous[3])):
+                normal_list.append(normal)
+                tmp_t.append(ct)
+                previous = [x,y,z,r]
+        tmp_t.append(t_list[-1])
+        #plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_t)
+        """
+        if len(tmp_t) > 2:
+            tmp_tmp_t = [tmp_t[0]]
+            for ct in tqdm(tmp_t[1:-1],position=2,desc="Cleaning Consecutive Colliding Contours"):
+                _,_,_,r = splev(ct,interp_xyz[kdx][0])
+                if contour_check(interp_xyz[kdx],interp_r[kdx],tmp_tmp_t[-1],ct,r*0.25,n_sides=20):
+                    if len(tmp_tmp_t) > 1:
+                        tmp_tmp_t[-1] = (ct+tmp_tmp_t[-1])/2
+                else:
+                    tmp_tmp_t.append(ct)
+            tmp_tmp_t.append(tmp_t[-1])
+        else:
+            tmp_tmp_t = tmp_t
+        """
+        tmp_tmp_t = tmp_t
+        #plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_tmp_t)
+        if len(tmp_t) > 2:
+            tmp_tmp_tmp_t = [tmp_tmp_t[0]]
+            desc3 = 'Cleaning remaining collisions'
+            desc3 = desc3+' '*(40-len(desc3))
+            for ct in tqdm(tmp_tmp_t[1:-1],bar_format='{l_bar}{bar:10}{r_bar}{bar:-10b}',position=3,leave=False,desc=desc3):
+                _,_,_,r = splev(ct,interp_xyz[kdx][0])
+                include = True
+                for cct in tmp_tmp_t[1:-1]:
+                    if cct==ct:
+                        continue
+                    if contour_check(interp_xyz[kdx],interp_r[kdx],ct,cct,r*0.25,n_sides=20):
+                        include = False
+                        break
+                if include:
+                    tmp_tmp_tmp_t.append(ct)
+            tmp_tmp_tmp_t.append(tmp_t[-1])
+        else:
+            tmp_tmp_tmp_t = tmp_tmp_t
+        #plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_tmp_tmp_t)
+        if len(tmp_tmp_t) > 2:
+            repeat = True
+            old_count = len(tmp_tmp_tmp_t)
+            resample_count = 1
+            while repeat:
+                tmp_tmp_tmp_tmp_t = [tmp_tmp_tmp_t[0]]
+                normal_list = []
+                nx,ny,nz,_ = splev(0,interp_xyz[kdx][0],der=1)
+                normal     = np.array([nx,ny,nz])
+                normal     = normal/np.linalg.norm(normal)
+                normal_list.append(normal)
+                highlight = []
+                count = 1
+                swap_list = []
+                total = len(tmp_tmp_tmp_t[1:-1])
+                desc4 = 'Resampling {}'.format(resample_count)
+                desc4 = desc4+' '*(40-len(desc4))
+                with tqdm(total=total,bar_format='{l_bar}{bar:10}{r_bar}{bar:-10b}',position=3,leave=False,desc=desc4) as pbar:
+                    for ct_idx,ct in enumerate(tmp_tmp_tmp_t[1:-1]):
+                        t_next = tmp_tmp_tmp_t[ct_idx + 2]
+                        x,y,z,r = splev(ct,interp_xyz[kdx][0])
+                        nx,ny,nz,_ = splev(ct,interp_xyz[kdx][0],der=1)
+                        normal     = np.array([nx,ny,nz])
+                        normal     = normal/np.linalg.norm(normal)
+                        angle      = abs((np.arccos(np.dot(normal_list[-1],normal.T))/np.pi)*180)
+                        if angle > ANGLE_THRESHOLD+5:
+                            t_op,f_op = optimize_contour(tmp_tmp_tmp_tmp_t[-1],ct,interp_xyz[kdx],interp_r[kdx],0.25,n_sides=20)
+                            tnx,tny,tnz,_ = splev(t_op,interp_xyz[kdx][0],der=1)
+                            next_nx,next_ny,next_nz,_ = splev(t_next,interp_xyz[kdx][0],der=1)
+                            t_op_normal     = np.array([tnx,tny,tnz])
+                            t_op_normal     = t_op_normal/np.linalg.norm(t_op_normal)
+                            angle0      = abs((np.arccos(np.dot(normal_list[-1],t_op_normal.T))/np.pi)*180)
+                            t_next_normal     = np.array([next_nx,next_ny,next_nz])
+                            t_next_normal     = t_next_normal/np.linalg.norm(t_next_normal)
+                            angle1      = abs((np.arccos(np.dot(t_op_normal,t_next_normal.T))/np.pi)*180)
+                            if f_op < 0 and (angle0 > ANGLE_THRESHOLD*0.5 and angle1 > ANGLE_THRESHOLD*0.5):
+                                #t_op = (tmp_tmp_tmp_t[-1]+ct)/2
+                                tmp_tmp_tmp_tmp_t.append(t_op)
+                                if count+1 < len(tmp_tmp_tmp_t)-1:
+                                    swap_list.append(count+1)
+                                if count-1 > 0:
+                                    swap_list.append(count-1)
+                                highlight.append(count)
+                                count += 1
+                            tmp_tmp_tmp_tmp_t.append(ct)
+                            count += 1
+                        else:
+                            tmp_tmp_tmp_tmp_t.append(ct)
+                            count += 1
+                        normal_list.append(normal)
+                        pbar.update(1)
+                tmp_tmp_tmp_tmp_t.append(tmp_tmp_t[-1])
+                #plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_tmp_tmp_tmp_t,highlight=highlight)
+                if len(tmp_tmp_tmp_tmp_t) == old_count:
+                    repeat = False
+                #else:
+                #    better_t,better_idx = swap_contour(interp_xyz[kdx],tmp_tmp_tmp_tmp_t,0.25)
+                #    tmp_tmp_tmp_tmp_t[better_idx] = better_t
+                #    plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_tmp_tmp_tmp_t,highlight=[better_idx])
+                for sw in swap_list:
+                    tmp_tmp_tmp_tmp_t[sw] = swap_contour(interp_xyz[kdx],tmp_tmp_tmp_tmp_t,sw,0.25)
+                #plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_tmp_tmp_tmp_t,highlight=swap_list)
+                tmp_tmp_tmp_tmp_t,better_idx = swap_worst_contour(interp_xyz[kdx],tmp_tmp_tmp_tmp_t,0.25,thresh=2*ANGLE_THRESHOLD)
+                #plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_tmp_tmp_tmp_t,highlight=better_idx)
+                old_count = len(tmp_tmp_tmp_tmp_t)
+                tmp_tmp_tmp_t = tmp_tmp_tmp_tmp_t
+                resample_count += 1
+        else:
+            tmp_tmp_tmp_tmp_t = tmp_tmp_tmp_t
+        #plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_tmp_tmp_tmp_t,highlight=highlight)
+        sample_t.append(tmp_tmp_tmp_tmp_t)
+    return sample_t
+"""
+ORIGINAL CONTOUR CHECKING
 def contour_check_all(interp_xyz,interp_r,radius_buffer):
     sample_t = []
     n = np.linspace(0,1,200)
@@ -584,7 +796,7 @@ def contour_check_all(interp_xyz,interp_r,radius_buffer):
                 #tmp_tmp_angles = [np.dot(normals[i,:],normals[i+1,:]) for i in range(normals.shape[0]-1)]
                 for jdx in range(collision_free[idx+1] - collision_free[idx]):
                     #max_diff = np.argmax(np.diff(tmp_tmp_angles))
-                    nx,ny,nz = splev(tmp_tmp_t,interp_xyz[kdx][0],der=1)
+                    nx,ny,nz,_ = splev(tmp_tmp_t,interp_xyz[kdx][0],der=1)
                     normals = np.array([nx,ny,nz]).T
                     tmp_tmp_angles = [np.dot(normals[i,:],normals[i+1,:]) for i in range(normals.shape[0]-1)]
                     max_diff = np.argmin(tmp_tmp_angles)
@@ -610,28 +822,29 @@ def contour_check_all(interp_xyz,interp_r,radius_buffer):
         sample_t.append(tmp_t)
     print('Total Collisions Resolved: '+str(total_collisions))
     return sample_t
+"""
 
 def sv_data(interp_xyz,interp_r,radius_buffer=0.25):
     points  = []
     radii   = []
     normals = []
     sample_t = contour_check_all(interp_xyz,interp_r,radius_buffer)
-    fig = plt.figure()
-    ax = fig.add_subplot(111,projection='3d')
+    #fig = plt.figure()
+    #ax = fig.add_subplot(111,projection='3d')
     for idx in range(len(interp_xyz)):
-        x,y,z    = splev(np.array(sample_t[idx]),interp_xyz[idx][0])
-        _,r        = splev(np.array(sample_t[idx]),interp_r[idx][0])
-        nx,ny,nz = splev(np.array(sample_t[idx]),interp_xyz[idx][0],der=1)
+        x,y,z,r    = splev(np.array(sample_t[idx]),interp_xyz[idx][0])
+        #_,r        = splev(np.array(sample_t[idx]),interp_r[idx][0])
+        nx,ny,nz,_ = splev(np.array(sample_t[idx]),interp_xyz[idx][0],der=1)
         l  = np.linalg.norm(np.array([nx,ny,nz]),axis=0)
         nx = nx/l
         ny = ny/l
         nz = nz/l
-        ax.plot3D(x,y,z)
-        for i in range(len(x)):
-            patch = Circle((0,0),r[i],facecolor='r')
-            ax.add_patch(patch)
-            pathpatch_2d_to_3d(patch,z=0,normal=np.array([nx[i],ny[i],nz[i]]))
-            pathpatch_translate(patch,(x[i],y[i],z[i]))
+        #ax.plot3D(x,y,z)
+        #for i in range(len(x)):
+        #    patch = Circle((0,0),r[i],facecolor='r')
+        #    ax.add_patch(patch)
+        #    pathpatch_2d_to_3d(patch,z=0,normal=np.array([nx[i],ny[i],nz[i]]))
+        #    pathpatch_translate(patch,(x[i],y[i],z[i]))
         tmp_points = np.zeros((len(x),3))
         tmp_points[:,0] = x
         tmp_points[:,1] = y
@@ -643,7 +856,7 @@ def sv_data(interp_xyz,interp_r,radius_buffer=0.25):
         tmp_normals[:,2] = nz
         radii.append(r.tolist())
         normals.append(tmp_normals.tolist())
-    plt.show()
+    #plt.show()
     return points,radii,normals
 
 def plot_frames(frames):
@@ -676,9 +889,9 @@ def plot_path_contours(interp_xyz,interp_r,t_list,highlight=[]):
     actors = []
     colors = vtk.vtkNamedColors()
     for t in t_list:
-        x,y,z = splev(t,interp_xyz[0])
-        _,r     = splev(t,interp_r[0])
-        nx,ny,nz = splev(t,interp_xyz[0],der=1)
+        x,y,z,r = splev(t,interp_xyz[0])
+        #_,r     = splev(t,interp_r[0])
+        nx,ny,nz,_ = splev(t,interp_xyz[0],der=1)
         points.InsertNextPoint([x,y,z])
         radii.append(r)
         normals.append([nx,ny,nz])
@@ -728,26 +941,91 @@ def plot_path_contours(interp_xyz,interp_r,t_list,highlight=[]):
     render_window.Render()
     interactor.Start()
 
-def optimize_contour(t0,t1,interp_xyz,interp_r,radius_buffer,t_buffer=0.1,n_sides=4):
-    nx0,ny0,nz0 = splev(t0,interp_xyz[0],der=1)
-    nx1,ny1,nz1 = splev(t1,interp_xyz[0],der=1)
+def optimize_contour(t0,t1,interp_xyz,interp_r,radius_buffer,t_buffer=0.1,n_sides=50):
+    nx0,ny0,nz0,_ = splev(t0,interp_xyz[0],der=1)
+    nx1,ny1,nz1,_ = splev(t1,interp_xyz[0],der=1)
     n0 = np.array([nx0,ny0,nz0])
     n1 = np.array([nx1,ny1,nz1])
     n0 = n0/np.linalg.norm(n0) #.reshape(-1,1)
     n1 = n1/np.linalg.norm(n1) #.reshape(-1,1)
     t_bound_buff = (t1-t0)*t_buffer
     def func(t,interp_xyz=interp_xyz,interp_r=interp_r,n0=n0,n1=n1,t0=t0,t1=t1):
-        nx,ny,nz = splev(t,interp_xyz[0],der=1)
+        x0,y0,z0,r0 = splev(t0,interp_xyz[0])
+        x1,y1,z1,r1 = splev(t1,interp_xyz[0])
+        x,y,z,r = splev(t,interp_xyz[0])
+        nx,ny,nz,_ = splev(t,interp_xyz[0],der=1)
         n = np.array([nx,ny,nz])
         n = n/np.linalg.norm(n) #.reshape(-1,1)
         #print(-np.dot(n0,n)*np.dot(n1,n))
-        a = np.dot(n0,n)
-        b = np.dot(n1,n)
+        n_avg = (n0+n1)/2
+        a = np.dot(n_avg,n)
+        first_angle = np.arccos(np.dot(n0,n))
+        second_angle = np.arccos(np.dot(n1,n))
+        #l0 = ((x0-x)**(2)+(y0-y)**(2)+(z0-z)**(2))**(1/2)
+        #l1 = ((x1-x)**(2)+(y1-y)**(2)+(z1-z)**(2))**(1/2)
+        mx  = (x0+x1)/2
+        my  = (y0+y1)/2
+        mz  = (z0+z1)/2
+        lm1 = ((x0-x)**(2)+(y0-y)**(2)+(z0-z)**(2))**(1/2)
+        lm2 = ((x1-x)**(2)+(y1-y)**(2)+(z1-z)**(2))**(1/2)
+        #d   = (((x1-x0)**(2)+(y1-y0)**(2)+(z1-z0)**(2))**(1/2))/2
+        #d   = d/2
+        lm = abs(lm1-lm2)/(lm1+lm2)
         c = 0
         if contour_check(interp_xyz,interp_r,t0,t,radius_buffer=radius_buffer,n_sides=n_sides):
             c = 100
         if contour_check(interp_xyz,interp_r,t1,t,radius_buffer=radius_buffer,n_sides=n_sides):
-            c = 100
-        return -(a*b+(1-abs(a-b))) + c
+            c += 100
+        if np.sin(first_angle)*(r0+r) > lm1:
+            c += 100
+        if np.sin(second_angle)*(r1+r) > lm2:
+            c += 100
+        #return -(a*b+(1-abs(a-b))) + c # + abs(l0-l1)
+        return -(a) + c + lm/100
     result = optimize.minimize_scalar(func,bounds=(t0+t_bound_buff,t1-t_bound_buff),method='bounded')
     return result.x,result.fun
+
+def swap_worst_contour(interp_xyz,t_list,radius_buffer,thresh=20):
+    normal_list = []
+    nx,ny,nz,_ = splev(0,interp_xyz[0],der=1)
+    normal     = np.array([nx,ny,nz])
+    normal     = normal/np.linalg.norm(normal)
+    normal_list.append(normal)
+    #next_nx,next_ny,next_nz,_ = splev(t_list[1],interp_xyz[kdx][0],der=1)
+    #next_normal     = np.array([next_nx,next_ny,next_nz])
+    #next_normal     = next_normal/np.linalg.norm(next_normal)
+    #angle = abs((np.arccos(np.dot(normal_list[-1],next_normal.T))/np.pi)*180)
+    #scores = [angle]
+    scores = []
+    total = len(t_list[1:-1])
+    desc5 = 'Computing swap scores'
+    desc5 = desc5+' '*(40-len(desc5))
+    with tqdm(total=total,bar_format='{l_bar}{bar:10}{r_bar}{bar:-10b}',position=4,leave=False,desc=desc5) as pbar:
+        for idx,ct in enumerate(t_list[1:-1]):
+            next_t            = t_list[idx+2]
+            nx,ny,nz,_        = splev(ct,interp_xyz[0],der=1)
+            normal            = np.array([nx,ny,nz])
+            normal            = normal/np.linalg.norm(normal)
+            angle0            = abs((np.arccos(np.dot(normal_list[-1],normal.T))/np.pi)*180)
+            next_nx,next_ny,next_nz,_ = splev(next_t,interp_xyz[0],der=1)
+            t_next_normal     = np.array([next_nx,next_ny,next_nz])
+            t_next_normal     = t_next_normal/np.linalg.norm(t_next_normal)
+            angle1            = abs((np.arccos(np.dot(normal,t_next_normal.T))/np.pi)*180)
+            scores.append(angle0+angle1)
+            normal_list.append(normal)
+            pbar.update(1)
+    worst_idx = np.argwhere(np.array(scores)>thresh).flatten()+1
+    worst_idx = worst_idx.tolist()
+    for wi in worst_idx:
+        t   = t_list[wi]
+        t0,f0 = optimize_contour(t_list[wi-1],t_list[wi+1],interp_xyz,None,radius_buffer,t_buffer=0.1,n_sides=50)
+        if f0 < 0:
+            t_list[wi] = t0
+    return t_list,worst_idx
+
+def swap_contour(interp_xyz,t_list,c_idx,radius_buffer):
+    value = t_list[c_idx]
+    t0,f0 = optimize_contour(t_list[c_idx-1],t_list[c_idx+1],interp_xyz,None,radius_buffer,t_buffer=0.1,n_sides=50)
+    if f0 < 0:
+        value = t0
+    return value
